package cn.superdesk.uniorder.controller.invoice.common;

import cn.superdesk.libs.common.json.JsonUtils;
import cn.superdesk.uniorder.common.pagination.PageQueryParam;
import cn.superdesk.uniorder.common.rest.response.R;
import cn.superdesk.uniorder.controller.base.BaseController;
import cn.superdesk.uniorder.dao.entity.invoice.common.InvoiceTaxRate;
import cn.superdesk.uniorder.dto.param.invoice.common.InvoiceTaxRateParam;
import cn.superdesk.uniorder.dubbo.consumer.invoice.common.InvoiceTaxRateDubboConsumerService;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.ArrayList;
import java.util.List;


/**
 * @Description:
 * @Author: linjinyu
 * @Email linjinyu@avic-s.com
 * @CreateDate: 2021-07-23 15:30:45
 * @UpdateUser: linjinyu
 * @UpdateDate: 2021-07-23 15:30:45
 * @UpdateRemark: 修改内容
 * @Version: 1.0
 */
@Slf4j
@RestController
@RequestMapping("/invoiceTaxRate")
public class InvoiceTaxRateController extends BaseController {

	@Autowired
	private InvoiceTaxRateDubboConsumerService invoiceTaxRateDubboConsumerService;

	@ApiOperation(value = "分页查询")
	@RequestMapping(value = "list", method = RequestMethod.POST)
	@ResponseBody
	public R<List<InvoiceTaxRate>> pageQueryInvoiceTaxRates(@RequestBody PageQueryParam param) {

		log.info("pageQueryInvoiceTaxRate:\n{}", JsonUtils.toJson(param));

		R<List<InvoiceTaxRate>> result = new R<>();

		// TODO 参数合法性判定

		result = invoiceTaxRateDubboConsumerService.pageQuery(param.getConditions(), param.getPageNo(), param.getPageSize());

		return result;
	}

	@ApiOperation(value = "按id查询")
	@RequestMapping(value = "{id}", method = RequestMethod.GET)
	@ResponseBody
	public R<List<InvoiceTaxRate>> getById(@PathVariable("id") long id) {

		R<List<InvoiceTaxRate>> result = new R<>();

		// TODO 参数合法性判定

		InvoiceTaxRate entity = invoiceTaxRateDubboConsumerService.findInvoiceTaxRateById(id);

		List<InvoiceTaxRate> data = new ArrayList<>();
		data.add(entity);

		result.setData(data);

		return result;
	}

	@ApiOperation(value = "新增")
	@RequestMapping(value = "add", method = RequestMethod.POST)
	@ResponseBody
	public R<String> addInvoiceTaxRate(@RequestBody InvoiceTaxRateParam param) {

		log.info("addInvoiceTaxRate:\n{}", JsonUtils.toJson(param));


		// TODO 参数合法性判定

		invoiceTaxRateDubboConsumerService.addInvoiceTaxRate(param);

		return new R<>("200", "操作成功", true);
	}

	@ApiOperation(value = "更新")
	@RequestMapping(value = "update", method = RequestMethod.POST)
	@ResponseBody
	public R<String> updateInvoiceTaxRate(@RequestBody InvoiceTaxRateParam param) {

		log.info("updateInvoiceTaxRate:\n{}", JsonUtils.toJson(param));

		invoiceTaxRateDubboConsumerService.updateInvoiceTaxRate(param);

		return new R<>("200", "操作成功", true);
	}

	@ApiOperation(value = "删除")
	@RequestMapping(value = "delete/{id}", method = RequestMethod.POST)
	@ResponseBody
	public R<String> deleteInvoiceTaxRate(@PathVariable("id") long id) {

		// TODO 参数合法性判定

		invoiceTaxRateDubboConsumerService.deleteInvoiceTaxRate(id);

		return new R<>("200", "操作成功", true);
	}

	@ApiOperation(value = "批量删除")
	@RequestMapping(value = "delete_ids", method = RequestMethod.POST)
	@ResponseBody
	public R<String> deleteIdsInvoiceTaxRate(@RequestParam(value = "ids") long[] ids) {

		log.info(JsonUtils.toJson(ids));

		for (long id : ids) {
			invoiceTaxRateDubboConsumerService.deleteInvoiceTaxRate(id);
		}

		return new R<>("200", "操作成功", true);

	}


}
