package cn.superdesk.uniorder.controller.marketing;

import cn.superdesk.libs.common.json.JsonUtils;
import cn.superdesk.uniorder.common.pagination.PageQueryGenericParam;
import cn.superdesk.uniorder.common.pagination.PageQueryParam;
import cn.superdesk.uniorder.common.rest.response.R;
import cn.superdesk.uniorder.controller.base.BaseController;
import cn.superdesk.uniorder.dao.entity.marketing.UnifiedStatisticsVirt;
import cn.superdesk.uniorder.dto.param.marketing.UnifiedStatisticsVirtDimDayQueryCriteria;
import cn.superdesk.uniorder.dto.param.marketing.UnifiedStatisticsVirtParam;
import cn.superdesk.uniorder.dto.param.marketing.UnifiedStatisticsVirtQueryCriteria;
import cn.superdesk.uniorder.dubbo.consumer.marketing.UnifiedStatisticsVirtDimMonthDubboConsumerService;
import cn.superdesk.uniorder.executor.scheduler.task.StatisticsJobVirtOrderStatisticalDimMonthTask;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.ArrayList;
import java.util.List;


/**
 * @Description:
 * @Author: linjinyu
 * @Email linjinyu@avic-s.com
 * @CreateDate: 2021-09-06 12:00:09
 * @UpdateUser: linjinyu
 * @UpdateDate: 2021-09-06 12:00:09
 * @UpdateRemark: 修改内容
 * @Version: 1.0
 */
@Slf4j
@RestController
@RequestMapping("/unifiedStatisticsVirtDimMonth")
public class UnifiedStatisticsVirtDimMonthController extends BaseController {

	@Autowired
	private UnifiedStatisticsVirtDimMonthDubboConsumerService unifiedStatisticsVirtDubboConsumerService;

	/****************************************** 手动执行任务 start ******************************************/
	@Autowired
	private StatisticsJobVirtOrderStatisticalDimMonthTask statisticsJobVirtOrderStatisticalDimMonthTask;

	/**
	 * http://192.168.6.223:8089/web/unifiedStatisticsVirtDimMonth/task
	 *
	 * @param criteria
	 * @return
	 */
	@ApiOperation(value = "手动执行任务-指定日期")
	@RequestMapping(value = "task", method = RequestMethod.POST)
	@ResponseBody
	public R<String> task(@RequestBody UnifiedStatisticsVirtDimDayQueryCriteria criteria) {

		// TODO 参数合法性判定

		statisticsJobVirtOrderStatisticalDimMonthTask.runTask(criteria.getYear(), criteria.getMonth());

		return new R<>("200", "操作成功", true);
	}

	/****************************************** 手动执行任务   end ******************************************/

	@ApiOperation(value = "分页查询-默认不用")
	@RequestMapping(value = "listByObscure", method = RequestMethod.POST)
	@ResponseBody
	public R<List<UnifiedStatisticsVirt>> pageQueryUnifiedStatisticsVirt(@RequestBody PageQueryParam param) {

		log.info("pageQueryUnifiedStatisticsVirt:\n{}", JsonUtils.toJson(param));

		R<List<UnifiedStatisticsVirt>> result = new R<>();

		// TODO 参数合法性判定

		result = unifiedStatisticsVirtDubboConsumerService.pageQuery(param.getConditions(), param.getPageNo(), param.getPageSize());

		return result;
	}

	@ApiOperation(value = "分页查询")
	@RequestMapping(value = "list", method = RequestMethod.POST)
	@ResponseBody
	public R<List<UnifiedStatisticsVirt>> pageQueryGenericParamUnifiedStatisticsVirt(@RequestBody PageQueryGenericParam<UnifiedStatisticsVirtQueryCriteria> param) {

		log.info("pageQueryUnifiedStatisticsVirt:\n{}", JsonUtils.toJson(param));

		R<List<UnifiedStatisticsVirt>> result = new R<>();

		// TODO 参数合法性判定

		result = unifiedStatisticsVirtDubboConsumerService.pageQueryGenericParam(param.getConditions());

		return result;
	}

	@ApiOperation(value = "按id查询")
	@RequestMapping(value = "{id}", method = RequestMethod.GET)
	@ResponseBody
	public R<List<UnifiedStatisticsVirt>> getById(@PathVariable("id") long id) {

		R<List<UnifiedStatisticsVirt>> result = new R<>();

		// TODO 参数合法性判定

		UnifiedStatisticsVirt entity = unifiedStatisticsVirtDubboConsumerService.findUnifiedStatisticsVirtById(id);

		List<UnifiedStatisticsVirt> data = new ArrayList<>();
		data.add(entity);

		result.setData(data);

		return result;
	}

	@ApiOperation(value = "新增")
	@RequestMapping(value = "add", method = RequestMethod.POST)
	@ResponseBody
	public R<String> addUnifiedStatisticsVirt(@RequestBody UnifiedStatisticsVirtParam param) {

		log.info("addUnifiedStatisticsVirt:\n{}", JsonUtils.toJson(param));


		// TODO 参数合法性判定

		unifiedStatisticsVirtDubboConsumerService.addUnifiedStatisticsVirt(param);

		return new R<>("200", "操作成功", true);
	}

	@ApiOperation(value = "更新")
	@RequestMapping(value = "update", method = RequestMethod.POST)
	@ResponseBody
	public R<String> updateUnifiedStatisticsVirt(@RequestBody UnifiedStatisticsVirtParam param) {

		log.info("updateUnifiedStatisticsVirt:\n{}", JsonUtils.toJson(param));

		unifiedStatisticsVirtDubboConsumerService.updateUnifiedStatisticsVirt(param);

		return new R<>("200", "操作成功", true);
	}

	@ApiOperation(value = "删除")
	@RequestMapping(value = "delete/{id}", method = RequestMethod.POST)
	@ResponseBody
	public R<String> deleteUnifiedStatisticsVirt(@PathVariable("id") long id) {

		// TODO 参数合法性判定

		unifiedStatisticsVirtDubboConsumerService.deleteUnifiedStatisticsVirt(id);

		return new R<>("200", "操作成功", true);
	}

	@ApiOperation(value = "批量删除")
	@RequestMapping(value = "delete_ids", method = RequestMethod.POST)
	@ResponseBody
	public R<String> deleteIdsUnifiedStatisticsVirt(@RequestParam(value = "ids") long[] ids) {

		log.info(JsonUtils.toJson(ids));

		for (long id : ids) {
			unifiedStatisticsVirtDubboConsumerService.deleteUnifiedStatisticsVirt(id);
		}

		return new R<>("200", "操作成功", true);

	}


}
