package cn.superdesk.uniorder.controller.scheduler;

import cn.superdesk.uniorder.common.json.AnJsonUtils;
import cn.superdesk.uniorder.common.pagination.PageQueryParam;
import cn.superdesk.uniorder.common.rest.response.R;
import cn.superdesk.uniorder.controller.base.BaseController;
import cn.superdesk.uniorder.dao.entity.scheduler.SysQuartzJob;
import cn.superdesk.uniorder.dto.param.scheduler.SysQuartzJobParam;
import cn.superdesk.uniorder.dubbo.consumer.scheduler.SysQuartzJobDubboConsumerService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.ArrayList;
import java.util.List;


/**
 * @Description: 定时任务
 * @Author: linjinyu
 * @Email linjinyu@avic-s.com
 * @CreateDate: 2020-12-17 16:35:26
 * @UpdateUser: linjinyu
 * @UpdateDate: 2020-12-17 16:35:26
 * @UpdateRemark: 修改内容
 * @Version: 1.0
 */
@Api(value = "定时任务-定义", tags = {"定时任务-定义"})
@Slf4j
@RestController
@RequestMapping("/sysQuartzJob")
public class SysQuartzJobController extends BaseController {

	@Autowired
	private SysQuartzJobDubboConsumerService sysQuartzJobDubboConsumerService;

	@ApiOperation(value = "分页查询")
	@RequestMapping(value = "list", method = RequestMethod.POST)
	@ResponseBody
	public R<List<SysQuartzJob>> pageQuerySysQuartzJobs(@RequestBody PageQueryParam param) {

		log.info("pageQuerySysQuartzJob:\n{}", AnJsonUtils.toJson(param));

		R<List<SysQuartzJob>> result = new R<>();

		// TODO 参数合法性判定

		result = sysQuartzJobDubboConsumerService.pageQuery(param.getConditions(), param.getPageNo(), param.getPageSize());

		return result;
	}

	@ApiOperation(value = "按id查询")
	@RequestMapping(value = "{id}", method = RequestMethod.GET)
	@ResponseBody
	public R<List<SysQuartzJob>> getById(@PathVariable("id") long id) {

		R<List<SysQuartzJob>> result = new R<>();

		// TODO 参数合法性判定

		SysQuartzJob entity = sysQuartzJobDubboConsumerService.findSysQuartzJobById(id);

		List<SysQuartzJob> data = new ArrayList<>();
		data.add(entity);

		result.setData(data);

		return result;
	}

	@ApiOperation(value = "新增")
	@RequestMapping(value = "add", method = RequestMethod.POST)
	@ResponseBody
	public R<String> addSysQuartzJob(@RequestBody SysQuartzJobParam param) {

		log.info("addSysQuartzJob:\n{}", AnJsonUtils.toJson(param));


		// TODO 参数合法性判定

		sysQuartzJobDubboConsumerService.addSysQuartzJob(param);

		return new R<>("200", "操作成功", true);
	}

	@ApiOperation(value = "更新")
	@RequestMapping(value = "update", method = RequestMethod.POST)
	@ResponseBody
	public R<String> updateSysQuartzJob(@RequestBody SysQuartzJobParam param) {

		log.info("updateSysQuartzJob:\n{}", AnJsonUtils.toJson(param));

		sysQuartzJobDubboConsumerService.updateSysQuartzJob(param);

		return new R<>("200", "操作成功", true);
	}

	@ApiOperation(value = "删除")
	@RequestMapping(value = "delete/{id}", method = RequestMethod.POST)
	@ResponseBody
	public R<String> deleteSysQuartzJob(@PathVariable("id") long id) {

		// TODO 参数合法性判定

		sysQuartzJobDubboConsumerService.deleteSysQuartzJob(id);

		return new R<>("200", "操作成功", true);
	}

	@ApiOperation(value = "批量删除")
	@RequestMapping(value = "delete_ids", method = RequestMethod.POST)
	@ResponseBody
	public R<String> deleteIdsSysQuartzJob(@RequestParam(value = "ids") long[] ids) {

		log.info(AnJsonUtils.toJson(ids));

		for (long id : ids) {
			sysQuartzJobDubboConsumerService.deleteSysQuartzJob(id);
		}

		return new R<>("200", "操作成功", true);

	}


}
