package cn.superdesk.uniorder.controller.scheduler;

import cn.superdesk.uniorder.common.json.AnJsonUtils;
import cn.superdesk.uniorder.common.pagination.PageQueryParam;
import cn.superdesk.uniorder.common.rest.response.R;
import cn.superdesk.uniorder.controller.base.BaseController;
import cn.superdesk.uniorder.dao.entity.scheduler.SysQuartzLog;
import cn.superdesk.uniorder.dto.param.scheduler.SysQuartzLogParam;
import cn.superdesk.uniorder.dubbo.consumer.scheduler.SysQuartzLogDubboConsumerService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.ArrayList;
import java.util.List;


/**
 * @Description: 定时任务日志
 * @Author: linjinyu
 * @Email linjinyu@avic-s.com
 * @CreateDate: 2020-12-17 16:35:26
 * @UpdateUser: linjinyu
 * @UpdateDate: 2020-12-17 16:35:26
 * @UpdateRemark: 修改内容
 * @Version: 1.0
 */
@Api(value = "定时任务-日志", tags = {"定时任务-日志"})
@Slf4j
@RestController
@RequestMapping("/sysQuartzLog")
public class SysQuartzLogController extends BaseController {

	@Autowired
	private SysQuartzLogDubboConsumerService sysQuartzLogDubboConsumerService;

	@ApiOperation(value = "分页查询")
	@RequestMapping(value = "list", method = RequestMethod.POST)
	@ResponseBody
	public R<List<SysQuartzLog>> pageQuerySysQuartzLogs(@RequestBody PageQueryParam param) {

		log.info("pageQuerySysQuartzLog:\n{}", AnJsonUtils.toJson(param));

		R<List<SysQuartzLog>> result = new R<>();

		// TODO 参数合法性判定

		result = sysQuartzLogDubboConsumerService.pageQuery(param.getConditions(), param.getPageNo(), param.getPageSize());

		return result;
	}

	@ApiOperation(value = "按id查询")
	@RequestMapping(value = "{id}", method = RequestMethod.GET)
	@ResponseBody
	public R<List<SysQuartzLog>> getById(@PathVariable("id") long id) {

		R<List<SysQuartzLog>> result = new R<>();

		// TODO 参数合法性判定

		SysQuartzLog entity = sysQuartzLogDubboConsumerService.findSysQuartzLogById(id);

		List<SysQuartzLog> data = new ArrayList<>();
		data.add(entity);

		result.setData(data);

		return result;
	}

	@ApiOperation(value = "新增")
	@RequestMapping(value = "add", method = RequestMethod.POST)
	@ResponseBody
	public R<String> addSysQuartzLog(@RequestBody SysQuartzLogParam param) {

		log.info("addSysQuartzLog:\n{}", AnJsonUtils.toJson(param));


		// TODO 参数合法性判定

		sysQuartzLogDubboConsumerService.addSysQuartzLog(param);

		return new R<>("200", "操作成功", true);
	}

	@ApiOperation(value = "更新")
	@RequestMapping(value = "update", method = RequestMethod.POST)
	@ResponseBody
	public R<String> updateSysQuartzLog(@RequestBody SysQuartzLogParam param) {

		log.info("updateSysQuartzLog:\n{}", AnJsonUtils.toJson(param));

		sysQuartzLogDubboConsumerService.updateSysQuartzLog(param);

		return new R<>("200", "操作成功", true);
	}

	@ApiOperation(value = "删除")
	@RequestMapping(value = "delete/{id}", method = RequestMethod.POST)
	@ResponseBody
	public R<String> deleteSysQuartzLog(@PathVariable("id") long id) {

		// TODO 参数合法性判定

		sysQuartzLogDubboConsumerService.deleteSysQuartzLog(id);

		return new R<>("200", "操作成功", true);
	}

	@ApiOperation(value = "批量删除")
	@RequestMapping(value = "delete_ids", method = RequestMethod.POST)
	@ResponseBody
	public R<String> deleteIdsSysQuartzLog(@RequestParam(value = "ids") long[] ids) {

		log.info(AnJsonUtils.toJson(ids));

		for (long id : ids) {
			sysQuartzLogDubboConsumerService.deleteSysQuartzLog(id);
		}

		return new R<>("200", "操作成功", true);

	}


}
