package cn.superdesk.uniorder.controller.sys;

import cn.superdesk.uniorder.common.json.AnJsonUtils;
import cn.superdesk.uniorder.common.pagination.PageQueryParam;
import cn.superdesk.uniorder.common.rest.response.R;
import cn.superdesk.uniorder.controller.base.BaseController;
import cn.superdesk.uniorder.dao.entity.sys.SysConfig;
import cn.superdesk.uniorder.dto.param.system.SysConfigParam;
import cn.superdesk.uniorder.dubbo.consumer.sys.SysConfigDubboConsumerService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.ArrayList;
import java.util.List;


/**
 * @Description:
 * @Author: linjinyu
 * @Email linjinyu@avic-s.com
 * @CreateDate: 2020-12-07 11:09:44
 * @UpdateUser: linjinyu
 * @UpdateDate: 2020-12-07 11:09:44
 * @UpdateRemark: 修改内容
 * @Version: 1.0
 */
@Api(value = "系统管理-系统配置", tags = {"系统管理-系统配置"})
@Slf4j
@RestController
@RequestMapping("/sysConfig")
public class SysConfigController extends BaseController {

	@Autowired
	private SysConfigDubboConsumerService sysConfigDubboConsumerService;

	@ApiOperation(value = "分页查询")
	@RequestMapping(value = "list", method = RequestMethod.POST)
	@ResponseBody
	public R<List<SysConfig>> pageQuerySysConfigs(@RequestBody PageQueryParam param) {

		log.info("pageQuerySysConfig:\n{}", AnJsonUtils.toJson(param));

		R<List<SysConfig>> result = new R<>();

		// TODO 参数合法性判定

		result = sysConfigDubboConsumerService.pageQuery(param.getConditions(), param.getPageNo(), param.getPageSize());

		return result;
	}

	@ApiOperation(value = "按id查询")
	@RequestMapping(value = "{id}", method = RequestMethod.GET)
	@ResponseBody
	public R<List<SysConfig>> getById(@PathVariable("id") long id) {

		R<List<SysConfig>> result = new R<>();

		// TODO 参数合法性判定

		SysConfig entity = sysConfigDubboConsumerService.findSysConfigById(id);

		List<SysConfig> data = new ArrayList<>();
		data.add(entity);

		result.setData(data);

		return result;
	}

	@ApiOperation(value = "新增")
	@RequestMapping(value = "add", method = RequestMethod.POST)
	@ResponseBody
	public R<String> addSysConfig(@RequestBody SysConfigParam param) {

		log.info("addSysConfig:\n{}", AnJsonUtils.toJson(param));


		// TODO 参数合法性判定

		sysConfigDubboConsumerService.addSysConfig(param);

		return new R<>("200", "操作成功", true);
	}

	@ApiOperation(value = "更新")
	@RequestMapping(value = "update", method = RequestMethod.POST)
	@ResponseBody
	public R<String> updateSysConfig(@RequestBody SysConfigParam param) {

		log.info("updateSysConfig:\n{}", AnJsonUtils.toJson(param));

		sysConfigDubboConsumerService.updateSysConfig(param);

		return new R<>("200", "操作成功", true);
	}

	@ApiOperation(value = "删除")
	@RequestMapping(value = "delete/{id}", method = RequestMethod.POST)
	@ResponseBody
	public R<String> deleteSysConfig(@PathVariable("id") long id) {

		// TODO 参数合法性判定

		sysConfigDubboConsumerService.deleteSysConfig(id);

		return new R<>("200", "操作成功", true);
	}

	@ApiOperation(value = "批量删除")
	@RequestMapping(value = "delete_ids", method = RequestMethod.POST)
	@ResponseBody
	public R<String> deleteIdsSysConfig(@RequestParam(value = "ids") long[] ids) {

		log.info(AnJsonUtils.toJson(ids));

		for (long id : ids) {
			sysConfigDubboConsumerService.deleteSysConfig(id);
		}

		return new R<>("200", "操作成功", true);

	}


}
