package cn.superdesk.uniorder.controller.sys.logger;

import cn.superdesk.libs.common.json.JsonUtils;
import cn.superdesk.uniorder.common.pagination.PageQueryParam;
import cn.superdesk.uniorder.common.rest.response.R;
import cn.superdesk.uniorder.controller.base.BaseController;
import cn.superdesk.uniorder.dao.entity.sys.logger.SysMonitorFunctionLog;
import cn.superdesk.uniorder.dto.param.system.logger.SysMonitorFunctionLogParam;
import cn.superdesk.uniorder.dubbo.consumer.sys.logger.SysMonitorFunctionLogDubboConsumerService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.ArrayList;
import java.util.List;


/**
 * @Description: 监控日志
 * @Author: linjinyu
 * @Email linjinyu@avic-s.com
 * @CreateDate: 2021-07-05 16:35:25
 * @UpdateUser: linjinyu
 * @UpdateDate: 2021-07-05 16:35:25
 * @UpdateRemark: 修改内容
 * @Version: 1.0
 */
@Api(value = "系统管理-监控日志", tags = {"系统管理-监控日志"})
@Slf4j
@RestController
@RequestMapping("/sysMonitorFunctionLog")
public class SysMonitorFunctionLogController extends BaseController {

	@Autowired
	private SysMonitorFunctionLogDubboConsumerService sysMonitorFunctionLogDubboConsumerService;

	@ApiOperation(value = "分页查询")
	@RequestMapping(value = "list", method = RequestMethod.POST)
	@ResponseBody
	public R<List<SysMonitorFunctionLog>> pageQuerySysMonitorFunctionLogs(@RequestBody PageQueryParam param) {

		log.info("pageQuerySysMonitorFunctionLog:\n{}", JsonUtils.toJson(param));

		R<List<SysMonitorFunctionLog>> result = new R<>();

		// TODO 参数合法性判定

		result = sysMonitorFunctionLogDubboConsumerService.pageQuery(param.getConditions(), param.getPageNo(), param.getPageSize());

		return result;
	}

	@ApiOperation(value = "按id查询")
	@RequestMapping(value = "{id}", method = RequestMethod.GET)
	@ResponseBody
	public R<List<SysMonitorFunctionLog>> getById(@PathVariable("id") long id) {

		R<List<SysMonitorFunctionLog>> result = new R<>();

		// TODO 参数合法性判定

		SysMonitorFunctionLog entity = sysMonitorFunctionLogDubboConsumerService.findSysMonitorFunctionLogById(id);

		List<SysMonitorFunctionLog> data = new ArrayList<>();
		data.add(entity);

		result.setData(data);

		return result;
	}

	@ApiOperation(value = "新增")
	@RequestMapping(value = "add", method = RequestMethod.POST)
	@ResponseBody
	public R<String> addSysMonitorFunctionLog(@RequestBody SysMonitorFunctionLogParam param) {

		log.info("addSysMonitorFunctionLog:\n{}", JsonUtils.toJson(param));


		// TODO 参数合法性判定

		sysMonitorFunctionLogDubboConsumerService.addSysMonitorFunctionLog(param);

		return new R<>("200", "操作成功", true);
	}

	@ApiOperation(value = "更新")
	@RequestMapping(value = "update", method = RequestMethod.POST)
	@ResponseBody
	public R<String> updateSysMonitorFunctionLog(@RequestBody SysMonitorFunctionLogParam param) {

		log.info("updateSysMonitorFunctionLog:\n{}", JsonUtils.toJson(param));

		sysMonitorFunctionLogDubboConsumerService.updateSysMonitorFunctionLog(param);

		return new R<>("200", "操作成功", true);
	}

	@ApiOperation(value = "删除")
	@RequestMapping(value = "delete/{id}", method = RequestMethod.POST)
	@ResponseBody
	public R<String> deleteSysMonitorFunctionLog(@PathVariable("id") long id) {

		// TODO 参数合法性判定

		sysMonitorFunctionLogDubboConsumerService.deleteSysMonitorFunctionLog(id);

		return new R<>("200", "操作成功", true);
	}

	@ApiOperation(value = "批量删除")
	@RequestMapping(value = "delete_ids", method = RequestMethod.POST)
	@ResponseBody
	public R<String> deleteIdsSysMonitorFunctionLog(@RequestParam(value = "ids") long[] ids) {

		log.info(JsonUtils.toJson(ids));

		for (long id : ids) {
			sysMonitorFunctionLogDubboConsumerService.deleteSysMonitorFunctionLog(id);
		}

		return new R<>("200", "操作成功", true);

	}


}
