package cn.superdesk.uniorder.controller.sys.notify;

import cn.superdesk.uniorder.common.json.AnJsonUtils;
import cn.superdesk.uniorder.common.pagination.PageQueryParam;
import cn.superdesk.uniorder.common.rest.response.R;
import cn.superdesk.uniorder.controller.base.BaseController;
import cn.superdesk.uniorder.dao.entity.sys.notify.SysNotifyCategory;
import cn.superdesk.uniorder.dto.param.system.notify.SysNotifyCategoryParam;
import cn.superdesk.uniorder.dubbo.consumer.sys.notify.SysNotifyCategoryDubboConsumerService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.ArrayList;
import java.util.List;


/**
 * @Description:
 * @Author: linjinyu
 * @Email linjinyu@avic-s.com
 * @CreateDate: 2021-03-01 17:37:17
 * @UpdateUser: linjinyu
 * @UpdateDate: 2021-03-01 17:37:17
 * @UpdateRemark: 修改内容
 * @Version: 1.0
 */
@Api(value = "系统管理-通知-分类管理", tags = {"系统管理-通知-分类管理"})
@Slf4j
@RestController
@RequestMapping("/sysNotifyCategory")
public class SysNotifyCategoryController extends BaseController {

	@Autowired
	private SysNotifyCategoryDubboConsumerService sysNotifyCategoryDubboConsumerService;

	@ApiOperation(value = "分页查询")
	@RequestMapping(value = "list", method = RequestMethod.POST)
	@ResponseBody
	public R<List<SysNotifyCategory>> pageQuerySysNotifyCategorys(@RequestBody PageQueryParam param) {

		log.info("pageQuerySysNotifyCategory:\n{}", AnJsonUtils.toJson(param));

		R<List<SysNotifyCategory>> result = new R<>();

		// TODO 参数合法性判定

		result = sysNotifyCategoryDubboConsumerService.pageQuery(param.getConditions(), param.getPageNo(), param.getPageSize());

		return result;
	}

	@ApiOperation(value = "按id查询")
	@RequestMapping(value = "{id}", method = RequestMethod.GET)
	@ResponseBody
	public R<List<SysNotifyCategory>> getById(@PathVariable("id") long id) {

		R<List<SysNotifyCategory>> result = new R<>();

		// TODO 参数合法性判定

		SysNotifyCategory entity = sysNotifyCategoryDubboConsumerService.findSysNotifyCategoryById(id);

		List<SysNotifyCategory> data = new ArrayList<>();
		data.add(entity);

		result.setData(data);

		return result;
	}

	@ApiOperation(value = "新增")
	@RequestMapping(value = "add", method = RequestMethod.POST)
	@ResponseBody
	public R<String> addSysNotifyCategory(@RequestBody SysNotifyCategoryParam param) {

		log.info("addSysNotifyCategory:\n{}", AnJsonUtils.toJson(param));


		// TODO 参数合法性判定

		sysNotifyCategoryDubboConsumerService.addSysNotifyCategory(param);

		return new R<>("200", "操作成功", true);
	}

	@ApiOperation(value = "更新")
	@RequestMapping(value = "update", method = RequestMethod.POST)
	@ResponseBody
	public R<String> updateSysNotifyCategory(@RequestBody SysNotifyCategoryParam param) {

		log.info("updateSysNotifyCategory:\n{}", AnJsonUtils.toJson(param));

		sysNotifyCategoryDubboConsumerService.updateSysNotifyCategory(param);

		return new R<>("200", "操作成功", true);
	}

	@ApiOperation(value = "删除")
	@RequestMapping(value = "delete/{id}", method = RequestMethod.POST)
	@ResponseBody
	public R<String> deleteSysNotifyCategory(@PathVariable("id") long id) {

		// TODO 参数合法性判定

		sysNotifyCategoryDubboConsumerService.deleteSysNotifyCategory(id);

		return new R<>("200", "操作成功", true);
	}

	@ApiOperation(value = "批量删除")
	@RequestMapping(value = "delete_ids", method = RequestMethod.POST)
	@ResponseBody
	public R<String> deleteIdsSysNotifyCategory(@RequestParam(value = "ids") long[] ids) {

		log.info(AnJsonUtils.toJson(ids));

		for (long id : ids) {
			sysNotifyCategoryDubboConsumerService.deleteSysNotifyCategory(id);
		}

		return new R<>("200", "操作成功", true);

	}


}
