package cn.superdesk.uniorder.controller.sys.notify;

import cn.superdesk.uniorder.common.json.AnJsonUtils;
import cn.superdesk.uniorder.common.pagination.PageQueryParam;
import cn.superdesk.uniorder.common.rest.response.R;
import cn.superdesk.uniorder.controller.base.BaseController;
import cn.superdesk.uniorder.dao.entity.sys.notify.SysNotify;
import cn.superdesk.uniorder.dto.param.system.notify.SysNotifyParam;
import cn.superdesk.uniorder.dubbo.consumer.sys.notify.SysNotifyDubboConsumerService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;


/**
 * @Description:
 * @Author: linjinyu
 * @Email linjinyu@avic-s.com
 * @CreateDate: 2021-03-01 17:37:17
 * @UpdateUser: linjinyu
 * @UpdateDate: 2021-03-01 17:37:17
 * @UpdateRemark: 修改内容
 * @Version: 1.0
 */
@Api(value = "系统管理-通知-管理", tags = {"系统管理-通知-管理"})
@Slf4j
@RestController
@RequestMapping("/sysNotify")
public class SysNotifyController extends BaseController {

	@Autowired
	private SysNotifyDubboConsumerService sysNotifyDubboConsumerService;

	@ApiOperation(value = "分页查询")
	@RequestMapping(value = "list", method = RequestMethod.POST)
	@ResponseBody
	public R<List<SysNotify>> pageQuerySysNotifys(@RequestBody PageQueryParam param) {

		log.info("pageQuerySysNotify:\n{}", AnJsonUtils.toJson(param));

		R<List<SysNotify>> result = new R<>();

		// TODO 参数合法性判定

		Map<String, Object> conditions = param.getConditions();



		result = sysNotifyDubboConsumerService.pageQuery(conditions, param.getPageNo(), param.getPageSize(), getToken());

		return result;
	}

	@ApiOperation(value = "按id查询")
	@RequestMapping(value = "{id}", method = RequestMethod.GET)
	@ResponseBody
	public R<List<SysNotify>> getById(@PathVariable("id") long id) {

		R<List<SysNotify>> result = new R<>();

		// TODO 参数合法性判定

		SysNotify entity = sysNotifyDubboConsumerService.findSysNotifyById(id);

		List<SysNotify> data = new ArrayList<>();
		data.add(entity);

		result.setData(data);

		return result;
	}

	@ApiOperation(value = "新增")
	@RequestMapping(value = "add", method = RequestMethod.POST)
	@ResponseBody
	public R<String> addSysNotify(@RequestBody SysNotifyParam param) {

		log.info("addSysNotify:\n{}", AnJsonUtils.toJson(param));


		// TODO 参数合法性判定

		sysNotifyDubboConsumerService.addSysNotify(param);

		return new R<>("200", "操作成功", true);
	}

	@ApiOperation(value = "更新")
	@RequestMapping(value = "update", method = RequestMethod.POST)
	@ResponseBody
	public R<String> updateSysNotify(@RequestBody SysNotifyParam param) {

		log.info("updateSysNotify:\n{}", AnJsonUtils.toJson(param));

		sysNotifyDubboConsumerService.updateSysNotify(param);

		return new R<>("200", "操作成功", true);
	}

	@ApiOperation(value = "删除")
	@RequestMapping(value = "delete/{id}", method = RequestMethod.POST)
	@ResponseBody
	public R<String> deleteSysNotify(@PathVariable("id") long id) {

		// TODO 参数合法性判定

		sysNotifyDubboConsumerService.deleteSysNotify(id);

		return new R<>("200", "操作成功", true);
	}

	@ApiOperation(value = "批量删除")
	@RequestMapping(value = "delete_ids", method = RequestMethod.POST)
	@ResponseBody
	public R<String> deleteIdsSysNotify(@RequestParam(value = "ids") long[] ids) {

		log.info(AnJsonUtils.toJson(ids));

		for (long id : ids) {
			sysNotifyDubboConsumerService.deleteSysNotify(id);
		}

		return new R<>("200", "操作成功", true);

	}


}
