package cn.superdesk.uniorder.controller.sys.role;

import cn.superdesk.libs.common.json.JsonUtils;
import cn.superdesk.uniorder.common.pagination.PageQueryGenericParam;
import cn.superdesk.uniorder.common.pagination.PageQueryParam;
import cn.superdesk.uniorder.common.rest.response.R;
import cn.superdesk.uniorder.controller.base.BaseController;
import cn.superdesk.uniorder.dao.entity.sys.role.SysMenuOpenApi;
import cn.superdesk.uniorder.dto.param.system.role.SysMenuOpenApiParam;
import cn.superdesk.uniorder.dto.param.system.role.SysMenuOpenApiQueryCriteria;
import cn.superdesk.uniorder.dubbo.consumer.sys.role.SysMenuOpenApiDubboConsumerService;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.*;

import java.util.ArrayList;
import java.util.List;


/**
 * @Description: 一级菜单-二级菜单-按钮 X 开放接口
 * @Author: linjinyu
 * @Email linjinyu@avic-s.com
 * @CreateDate: 2021-11-01 10:58:18
 * @UpdateUser: linjinyu
 * @UpdateDate: 2021-11-01 10:58:18
 * @UpdateRemark: 修改内容
 * @Version: 1.0
 */
@Slf4j
@RestController
@RequestMapping("/sysMenuOpenApi")
public class SysMenuOpenApiController extends BaseController {

	@Autowired
	private SysMenuOpenApiDubboConsumerService sysMenuOpenApiDubboConsumerService;

	@ApiOperation(value = "分页查询 - 默认不用")
	@RequestMapping(value = "listByObscure", method = RequestMethod.POST)
	@ResponseBody
	public R<List<SysMenuOpenApi>> pageQuerySysMenuOpenApi(@RequestBody PageQueryParam param) {

		log.info("pageQuerySysMenuOpenApi:\n{}", JsonUtils.toJson(param));

		R<List<SysMenuOpenApi>> result = new R<>();

		// TODO 参数合法性判定

		result = sysMenuOpenApiDubboConsumerService.pageQuery(param.getConditions(), param.getPageNo(), param.getPageSize());

		return result;
	}

	@ApiOperation(value = "分页查询")
	@RequestMapping(value = "list", method = RequestMethod.POST)
	@ResponseBody
	public R<List<SysMenuOpenApi>> pageQueryGenericParamSysMenuOpenApi(@RequestBody PageQueryGenericParam<SysMenuOpenApiQueryCriteria> param) {

		log.info("pageQuerySysMenuOpenApi:\n{}", JsonUtils.toJson(param));

		R<List<SysMenuOpenApi>> result = new R<>();

		// TODO 参数合法性判定

		result = sysMenuOpenApiDubboConsumerService.pageQueryGenericParam(param.getConditions());

		return result;
	}

	@ApiOperation(value = "按id查询")
	@RequestMapping(value = "{id}", method = RequestMethod.GET)
	@ResponseBody
	public R<List<SysMenuOpenApi>> getById(@PathVariable("id") long id) {

		R<List<SysMenuOpenApi>> result = new R<>();

		// TODO 参数合法性判定

		SysMenuOpenApi entity = sysMenuOpenApiDubboConsumerService.findSysMenuOpenApiById(id);

		List<SysMenuOpenApi> data = new ArrayList<>();
		data.add(entity);

		result.setData(data);

		return result;
	}

	@ApiOperation(value = "新增")
	@RequestMapping(value = "add", method = RequestMethod.POST)
	@ResponseBody
	public R<String> addSysMenuOpenApi(@RequestBody SysMenuOpenApiParam param) {

		log.info("addSysMenuOpenApi:\n{}", JsonUtils.toJson(param));


		// TODO 参数合法性判定

		sysMenuOpenApiDubboConsumerService.addSysMenuOpenApi(param);

		return new R<>("200", "操作成功", true);
	}

	@ApiOperation(value = "更新")
	@RequestMapping(value = "update", method = RequestMethod.POST)
	@ResponseBody
	public R<String> updateSysMenuOpenApi(@RequestBody SysMenuOpenApiParam param) {

		log.info("updateSysMenuOpenApi:\n{}", JsonUtils.toJson(param));

		sysMenuOpenApiDubboConsumerService.updateSysMenuOpenApi(param);

		return new R<>("200", "操作成功", true);
	}

	@ApiOperation(value = "删除")
	@RequestMapping(value = "delete/{id}", method = RequestMethod.POST)
	@ResponseBody
	public R<String> deleteSysMenuOpenApi(@PathVariable("id") long id) {

		// TODO 参数合法性判定

		sysMenuOpenApiDubboConsumerService.deleteSysMenuOpenApi(id);

		return new R<>("200", "操作成功", true);
	}

	@ApiOperation(value = "批量删除")
	@RequestMapping(value = "delete_ids", method = RequestMethod.POST)
	@ResponseBody
	public R<String> deleteIdsSysMenuOpenApi(@RequestParam(value = "ids") long[] ids) {

		log.info(JsonUtils.toJson(ids));

		for (long id : ids) {
			sysMenuOpenApiDubboConsumerService.deleteSysMenuOpenApi(id);
		}

		return new R<>("200", "操作成功", true);

	}


}
