package cn.superdesk.uniorder.executor.scheduler.config.thread;

import lombok.extern.slf4j.Slf4j;
import org.springframework.aop.interceptor.AsyncUncaughtExceptionHandler;
import org.springframework.context.annotation.Configuration;
import org.springframework.scheduling.annotation.AsyncConfigurer;
import org.springframework.scheduling.concurrent.ThreadPoolTaskExecutor;

import java.util.concurrent.Executor;
import java.util.concurrent.ThreadPoolExecutor;

/**
 * @Description: 异步任务线程池装配类
 * @Author: linjinyu
 * @CreateDate: 2020/12/23 下午2:15
 * @UpdateUser: linjinyu
 * @UpdateDate: 2020/12/23 下午2:15
 * @UpdateRemark: 修改内容
 * @Version: 1.0
 */
@Slf4j
@Configuration
public class AsyncTaskExecutePool implements AsyncConfigurer {

	/**
	 * 注入配置类
	 */
	private final AsyncTaskProperties config;

	public AsyncTaskExecutePool(AsyncTaskProperties config) {
		this.config = config;
	}

	@Override
	public Executor getAsyncExecutor() {

		ThreadPoolTaskExecutor executor = new ThreadPoolTaskExecutor();
		// 核心线程池大小
		executor.setCorePoolSize(config.getCorePoolSize());
		// 最大线程数
		executor.setMaxPoolSize(config.getMaxPoolSize());
		// 队列容量
		executor.setQueueCapacity(config.getQueueCapacity());
		// 活跃时间
		executor.setKeepAliveSeconds(config.getKeepAliveSeconds());
		// 线程名字前缀
		executor.setThreadNamePrefix("el-async-");
		// setRejectedExecutionHandler:当pool已经达到max size的时候,如何处理新任务
		// CallerRunsPolicy:不在新线程中执行任务,而是由调用者所在的线程来执行
		executor.setRejectedExecutionHandler(new ThreadPoolExecutor.CallerRunsPolicy());
		executor.initialize();

		return executor;
	}

	@Override
	public AsyncUncaughtExceptionHandler getAsyncUncaughtExceptionHandler() {
		return (throwable, method, objects) -> {
			log.error("====" + throwable.getMessage() + "====", throwable);
			log.error("exception method:" + method.getName());
		};
	}
}
